# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._api_resource import APIResource
from stripe._stripe_object import StripeObject
from stripe._test_helpers import APIResourceTestHelpers
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Dict, Optional, cast, overload
from typing_extensions import Literal, Type, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.shared_payment._granted_token_create_params import (
        GrantedTokenCreateParams,
    )
    from stripe.params.shared_payment._granted_token_retrieve_params import (
        GrantedTokenRetrieveParams,
    )
    from stripe.params.shared_payment._granted_token_revoke_params import (
        GrantedTokenRevokeParams,
    )


class GrantedToken(APIResource["GrantedToken"]):
    """
    SharedPaymentGrantedToken is the view-only resource of a SharedPaymentIssuedToken, which is a limited-use reference to a PaymentMethod.
    When another Stripe merchant shares a SharedPaymentIssuedToken with you, you can view attributes of the shared token using the SharedPaymentGrantedToken API, and use it with a PaymentIntent.
    """

    OBJECT_NAME: ClassVar[Literal["shared_payment.granted_token"]] = (
        "shared_payment.granted_token"
    )

    class UsageDetails(StripeObject):
        class AmountCaptured(StripeObject):
            currency: str
            """
            Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
            """
            value: int
            """
            Integer value of the amount in the smallest currency unit.
            """

        amount_captured: Optional[AmountCaptured]
        """
        The total amount captured using this SharedPaymentToken.
        """
        _inner_class_types = {"amount_captured": AmountCaptured}

    class UsageLimits(StripeObject):
        currency: str
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """
        expires_at: int
        """
        Time at which this SharedPaymentToken expires and can no longer be used to confirm a PaymentIntent.
        """
        max_amount: int
        """
        Max amount that can be captured using this SharedPaymentToken.
        """

    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    deactivated_at: Optional[int]
    """
    Time at which this SharedPaymentGrantedToken expires and can no longer be used to confirm a PaymentIntent.
    """
    deactivated_reason: Optional[Literal["consumed", "expired", "revoked"]]
    """
    The reason why the SharedPaymentGrantedToken has been deactivated.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["shared_payment.granted_token"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    shared_metadata: Optional[Dict[str, str]]
    """
    Metadata about the SharedPaymentGrantedToken.
    """
    usage_details: Optional[UsageDetails]
    """
    Some details about how the SharedPaymentGrantedToken has been used already.
    """
    usage_limits: Optional[UsageLimits]
    """
    Limits on how this SharedPaymentGrantedToken can be used.
    """

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["GrantedTokenRetrieveParams"]
    ) -> "GrantedToken":
        """
        Retrieves an existing SharedPaymentGrantedToken object
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["GrantedTokenRetrieveParams"]
    ) -> "GrantedToken":
        """
        Retrieves an existing SharedPaymentGrantedToken object
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    class TestHelpers(APIResourceTestHelpers["GrantedToken"]):
        _resource_cls: Type["GrantedToken"]

        @classmethod
        def create(
            cls, **params: Unpack["GrantedTokenCreateParams"]
        ) -> "GrantedToken":
            """
            Creates a new test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to create SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens",
                    params=params,
                ),
            )

        @classmethod
        async def create_async(
            cls, **params: Unpack["GrantedTokenCreateParams"]
        ) -> "GrantedToken":
            """
            Creates a new test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to create SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens",
                    params=params,
                ),
            )

        @classmethod
        def _cls_revoke(
            cls,
            shared_payment_granted_token: str,
            **params: Unpack["GrantedTokenRevokeParams"],
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens/{shared_payment_granted_token}/revoke".format(
                        shared_payment_granted_token=sanitize_id(
                            shared_payment_granted_token
                        )
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def revoke(
            shared_payment_granted_token: str,
            **params: Unpack["GrantedTokenRevokeParams"],
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            ...

        @overload
        def revoke(
            self, **params: Unpack["GrantedTokenRevokeParams"]
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            ...

        @class_method_variant("_cls_revoke")
        def revoke(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["GrantedTokenRevokeParams"]
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens/{shared_payment_granted_token}/revoke".format(
                        shared_payment_granted_token=sanitize_id(
                            self.resource.get("id")
                        )
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_revoke_async(
            cls,
            shared_payment_granted_token: str,
            **params: Unpack["GrantedTokenRevokeParams"],
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens/{shared_payment_granted_token}/revoke".format(
                        shared_payment_granted_token=sanitize_id(
                            shared_payment_granted_token
                        )
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def revoke_async(
            shared_payment_granted_token: str,
            **params: Unpack["GrantedTokenRevokeParams"],
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            ...

        @overload
        async def revoke_async(
            self, **params: Unpack["GrantedTokenRevokeParams"]
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            ...

        @class_method_variant("_cls_revoke_async")
        async def revoke_async(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["GrantedTokenRevokeParams"]
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens/{shared_payment_granted_token}/revoke".format(
                        shared_payment_granted_token=sanitize_id(
                            self.resource.get("id")
                        )
                    ),
                    params=params,
                ),
            )

    @property
    def test_helpers(self):
        return self.TestHelpers(self)

    _inner_class_types = {
        "usage_details": UsageDetails,
        "usage_limits": UsageLimits,
    }


GrantedToken.TestHelpers._resource_cls = GrantedToken



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List
from typing_extensions import NotRequired


class FeatureCreateParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    lookup_key: str
    """
    A unique key you provide as your own system identifier. This may be up to 80 characters.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    name: str
    """
    The feature's name, for your own purpose, not meant to be displayable to the customer.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class SettlementAllocationIntentUpdateParams(TypedDict):
    amount: NotRequired["SettlementAllocationIntentUpdateParamsAmount"]
    """
    The new amount for the SettlementAllocationIntent. Only amount.value can be updated and currency must remain same.
    """
    reference: NotRequired[str]
    """
    The new reference for the SettlementAllocationIntent.
    """


class SettlementAllocationIntentUpdateParamsAmount(TypedDict):
    value: NotRequired[int]
    """
    A non-negative integer representing how much to charge in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
    """
    currency: NotRequired[str]
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2._billing_service import BillingService
    from stripe.v2._core_service import CoreService
    from stripe.v2._iam_service import IamService
    from stripe.v2._money_management_service import MoneyManagementService
    from stripe.v2._payment_service import PaymentService
    from stripe.v2._reporting_service import ReportingService
    from stripe.v2._tax_service import TaxService
    from stripe.v2._test_helper_service import TestHelperService

_subservices = {
    "billing": ["stripe.v2._billing_service", "BillingService"],
    "core": ["stripe.v2._core_service", "CoreService"],
    "iam": ["stripe.v2._iam_service", "IamService"],
    "money_management": [
        "stripe.v2._money_management_service",
        "MoneyManagementService",
    ],
    "payments": ["stripe.v2._payment_service", "PaymentService"],
    "reporting": ["stripe.v2._reporting_service", "ReportingService"],
    "tax": ["stripe.v2._tax_service", "TaxService"],
    "test_helpers": ["stripe.v2._test_helper_service", "TestHelperService"],
}


class V2Services(StripeService):
    billing: "BillingService"
    core: "CoreService"
    iam: "IamService"
    money_management: "MoneyManagementService"
    payments: "PaymentService"
    reporting: "ReportingService"
    tax: "TaxService"
    test_helpers: "TestHelperService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired


class AuthorizationRespondParams(RequestOptions):
    confirmed: bool
    """
    Whether to simulate the user confirming that the transaction was legitimate (true) or telling Stripe that it was fraudulent (false).
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class UsBankAccount(StripeObject):
    """
    Use the USBankAccounts API to create and manage US bank accounts objects that you can use to receive funds. Note that these are not interchangeable with v1 Tokens.
    """

    OBJECT_NAME: ClassVar[Literal["v2.core.vault.us_bank_account"]] = (
        "v2.core.vault.us_bank_account"
    )

    class AlternativeReference(StripeObject):
        id: str
        """
        The ID of the alternative resource being referenced.
        """
        type: Literal["external_account", "payment_method"]
        """
        The type of the alternative reference (e.g., external_account for V1 external accounts).
        """

    class Verification(StripeObject):
        class MicrodepositVerificationDetails(StripeObject):
            expires: str
            """
            Time when microdeposits will expire and have to be re-sent.
            """
            microdeposit_type: Literal["amounts", "descriptor_code"]
            """
            Microdeposit type can be amounts or descriptor_type.
            """
            sent: str
            """
            Time when microdeposits were sent.
            """

        microdeposit_verification_details: Optional[
            MicrodepositVerificationDetails
        ]
        """
        The microdeposit verification details if the status is awaiting verification.
        """
        status: Literal[
            "awaiting_verification",
            "unverified",
            "verification_failed",
            "verified",
        ]
        """
        The bank account verification status.
        """
        _inner_class_types = {
            "microdeposit_verification_details": MicrodepositVerificationDetails,
        }

    alternative_reference: Optional[AlternativeReference]
    """
    The alternative reference for this payout method, if it's a projected payout method.
    """
    archived: bool
    """
    Whether this USBankAccount object was archived.
    """
    bank_account_type: Literal["checking", "savings"]
    """
    Closed Enum. The type of bank account (checking or savings).
    """
    bank_name: str
    """
    The name of the bank this bank account belongs to. This field is populated automatically by Stripe based on the routing number.
    """
    created: str
    """
    Creation time of the object.
    """
    fedwire_routing_number: Optional[str]
    """
    The fedwire routing number of the bank account.
    """
    financial_connections_account: Optional[str]
    """
    The ID of the Financial Connections Account used to create the bank account.
    """
    id: str
    """
    The ID of the USBankAccount object.
    """
    last4: str
    """
    The last 4 digits of the account number.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.core.vault.us_bank_account"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    routing_number: Optional[str]
    """
    The ACH routing number of the bank account.
    """
    verification: Verification
    """
    The bank account verification details.
    """
    _inner_class_types = {
        "alternative_reference": AlternativeReference,
        "verification": Verification,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._api_mode import ApiMode
from stripe._stripe_object import StripeObject
from stripe._stripe_response import StripeResponse
from stripe.v2.core._event import Event, EventNotification
from typing import Any, Dict, List, Optional, cast
from typing_extensions import Literal, TYPE_CHECKING, override

if TYPE_CHECKING:
    from stripe._api_requestor import _APIRequestor


class V2CoreHealthApiErrorResolvedEventNotification(EventNotification):
    LOOKUP_TYPE = "v2.core.health.api_error.resolved"
    type: Literal["v2.core.health.api_error.resolved"]

    @override
    def fetch_event(self) -> "V2CoreHealthApiErrorResolvedEvent":
        return cast(
            "V2CoreHealthApiErrorResolvedEvent",
            super().fetch_event(),
        )

    @override
    async def fetch_event_async(self) -> "V2CoreHealthApiErrorResolvedEvent":
        return cast(
            "V2CoreHealthApiErrorResolvedEvent",
            await super().fetch_event_async(),
        )


class V2CoreHealthApiErrorResolvedEvent(Event):
    LOOKUP_TYPE = "v2.core.health.api_error.resolved"
    type: Literal["v2.core.health.api_error.resolved"]

    class V2CoreHealthApiErrorResolvedEventData(StripeObject):
        class Impact(StripeObject):
            class TopImpactedAccount(StripeObject):
                account: str
                """
                The account ID of the impacted connected account.
                """
                impacted_requests: int
                """
                The number of impacted requests.
                """
                impacted_requests_percentage: Optional[str]
                """
                The percentage of impacted requests.
                """

            canonical_path: str
            """
            The canonical path.
            """
            error_code: Optional[str]
            """
            The error code.
            """
            http_method: Literal["DELETE", "GET", "POST", "PUT"]
            """
            The HTTP method.
            """
            http_status: str
            """
            The HTTP status.
            """
            impacted_requests: int
            """
            The number of impacted requests.
            """
            impacted_requests_percentage: Optional[str]
            """
            The percentage of impacted requests.
            """
            top_impacted_accounts: Optional[List[TopImpactedAccount]]
            """
            The top impacted connected accounts (only for platforms).
            """
            _inner_class_types = {"top_impacted_accounts": TopImpactedAccount}

        grouping_key: str
        """
        The grouping key for the alert.
        """
        impact: Impact
        """
        The user impact.
        """
        resolved_at: str
        """
        The time when the user experience has returned to expected levels.
        """
        started_at: str
        """
        The time when impact on the user experience was first detected.
        """
        summary: str
        """
        A short description of the alert.
        """
        _inner_class_types = {"impact": Impact}

    data: V2CoreHealthApiErrorResolvedEventData
    """
    Data for the v2.core.health.api_error.resolved event
    """

    @classmethod
    def _construct_from(
        cls,
        *,
        values: Dict[str, Any],
        last_response: Optional[StripeResponse] = None,
        requestor: "_APIRequestor",
        api_mode: ApiMode,
    ) -> "V2CoreHealthApiErrorResolvedEvent":
        evt = super()._construct_from(
            values=values,
            last_response=last_response,
            requestor=requestor,
            api_mode=api_mode,
        )
        if hasattr(evt, "data"):
            evt.data = V2CoreHealthApiErrorResolvedEvent.V2CoreHealthApiErrorResolvedEventData._construct_from(
                values=evt.data,
                last_response=last_response,
                requestor=requestor,
                api_mode=api_mode,
            )
        return evt



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class FinancialAddressCreditParams(TypedDict):
    amount: "FinancialAddressCreditParamsAmount"
    """
    Object containing the amount value and currency to credit.
    """
    network: Literal["ach", "fps", "rtp", "sepa_credit_transfer", "wire"]
    """
    Open Enum. The network to use in simulating the funds flow. This will be the reflected in the resulting ReceivedCredit.
    """
    statement_descriptor: NotRequired[str]
    """
    String explaining funds flow. Use this field to populate the statement descriptor of the ReceivedCredit created as an eventual result of this simulation.
    """


class FinancialAddressCreditParamsAmount(TypedDict):
    value: NotRequired[int]
    """
    A non-negative integer representing how much to charge in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
    """
    currency: NotRequired[str]
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
